/*!
 * @file main.c
 * @brief UART MUX 4 Click Example.
 *
 * # Description
 * This example demonstrates the use of UART MUX 4 click board by processing
 * the incoming data and displaying them on the USB UART.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initializes the driver and switches the multiplexer position to enable UART1.
 *
 * ## Application Task
 * Writes demo message, echos it back, processes all incoming data 
 * and displays them on the USB UART.
 *
 * ## Additional Function
 * - static void uartmux4_clear_app_buf ( void )
 * - static err_t uartmux4_process ( uartmux4_t *ctx )
 *
 * @author Nenad Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "uartmux4.h"

#define PROCESS_BUFFER_SIZE 200
#define DEMO_MESSAGE "\r\nMikroE\r\n"

static uartmux4_t uartmux4;
static log_t logger;

static uint8_t app_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
static int32_t app_buf_len = 0;

/**
 * @brief UART MUX 4 clearing application buffer.
 * @details This function clears memory of application buffer and reset its length.
 * @note None.
 */
static void uartmux4_clear_app_buf ( void );

/**
 * @brief UART MUX 4 data reading function.
 * @details This function reads data from device and concatenates data to application buffer. 
 * @param[in] ctx : Click context object.
 * See #uartmux4_t object definition for detailed explanation.
 * @return @li @c  0 - Read some data.
 *         @li @c -1 - Nothing is read.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t uartmux4_process ( uartmux4_t *ctx );

void application_init ( void ) 
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    uartmux4_cfg_t uartmux4_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    uartmux4_cfg_setup( &uartmux4_cfg );
    UARTMUX4_MAP_MIKROBUS( uartmux4_cfg, MIKROBUS_1 );
    if ( UART_ERROR == uartmux4_init( &uartmux4, &uartmux4_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    uartmux4_enable_uart1( &uartmux4 );
    Delay_ms( 100 );
    
    log_info( &logger, " Application Task " );
    log_printf( &logger, " ---------------- \r\n" );
    Delay_ms( 100 );
}

void application_task ( void ) 
{
    if ( uartmux4_generic_write ( &uartmux4, DEMO_MESSAGE, sizeof( DEMO_MESSAGE ) ) )
    {
        if ( UARTMUX4_OK == uartmux4_process( &uartmux4 ) )
        {
            log_printf( &logger, "%s", app_buf );
            uartmux4_clear_app_buf( );
        }
    }
    Delay_ms( 2000 );
}

void main ( void ) 
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }
}

static void uartmux4_clear_app_buf ( void ) 
{
    memset( app_buf, 0, app_buf_len );
    app_buf_len = 0;
}

static err_t uartmux4_process ( uartmux4_t *ctx ) 
{
    uint8_t rx_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
    int32_t rx_size = 0;
    rx_size = uartmux4_generic_read( ctx, rx_buf, PROCESS_BUFFER_SIZE );
    if ( rx_size > 0 ) 
    {
        int32_t buf_cnt = app_buf_len;
        if ( ( ( app_buf_len + rx_size ) > PROCESS_BUFFER_SIZE ) && ( app_buf_len > 0 ) ) 
        {
            buf_cnt = PROCESS_BUFFER_SIZE - ( ( app_buf_len + rx_size ) - PROCESS_BUFFER_SIZE );
            memmove ( app_buf, &app_buf[ PROCESS_BUFFER_SIZE - buf_cnt ], buf_cnt );
        }
        for ( int32_t rx_cnt = 0; rx_cnt < rx_size; rx_cnt++ ) 
        {
            if ( rx_buf[ rx_cnt ] ) 
            {
                app_buf[ buf_cnt++ ] = rx_buf[ rx_cnt ];
                if ( app_buf_len < PROCESS_BUFFER_SIZE )
                {
                    app_buf_len++;
                }
            }
        }
        return UARTMUX4_OK;
    }
    return UARTMUX4_ERROR;
}

// ------------------------------------------------------------------------ END
